import graphql
import pytest

from dagster_graphql.test.utils import execute_dagster_graphql

from .setup import define_context
from .utils import sync_execute_get_events


def get_step_output(logs, step_key):
    for log in logs:
        if log['__typename'] == 'ExecutionStepOutputEvent' and log['step']['key'] == step_key:
            return log


def test_multi_mode_successful():
    add_mode_logs = sync_execute_get_events(
        {
            'executionParams': {
                'selector': {'name': 'multi_mode_with_resources'},
                'mode': 'add_mode',
                'environmentConfigData': {'resources': {'op': {'config': 2}}},
            }
        }
    )
    assert get_step_output(add_mode_logs, 'apply_to_three.compute')['valueRepr'] == '5'

    mult_mode_logs = sync_execute_get_events(
        {
            'executionParams': {
                'selector': {'name': 'multi_mode_with_resources'},
                'mode': 'mult_mode',
                'environmentConfigData': {'resources': {'op': {'config': 2}}},
            }
        }
    )
    assert get_step_output(mult_mode_logs, 'apply_to_three.compute')['valueRepr'] == '6'

    double_adder_mode_logs = sync_execute_get_events(
        {
            'executionParams': {
                'selector': {'name': 'multi_mode_with_resources'},
                'mode': 'double_adder',
                'environmentConfigData': {
                    'resources': {'op': {'config': {'num_one': 2, 'num_two': 4}}}
                },
            }
        }
    )
    assert get_step_output(double_adder_mode_logs, 'apply_to_three.compute')['valueRepr'] == '9'


MODE_QUERY = '''
query ModesQuery($pipelineName: String!, $mode: String!)
{
  pipeline(params: { name: $pipelineName }) {
    configTypes(mode: $mode) {
      name
    }
    environmentType(mode: $mode){
      name
      ... on CompositeConfigType {
        fields {
          configType {
            name
          }
        }
      }
    }
    presets {
      name
      mode
    }
    modes {
      name
      description
      resources {
        name
        configField {
          configType {
            name
            ... on CompositeConfigType {
              fields {
                name
                configType {
                  name
                }
              }
            }
          }
        }
      }
      loggers {
        name
        configField {
          configType {
            name
            ... on CompositeConfigType {
              fields {
                name
                configType {
                  name
                }
              }
            }
          }
        }
      }
    }
  }
}
'''


def execute_modes_query(pipeline_name, mode):
    return execute_dagster_graphql(
        define_context(), MODE_QUERY, variables={'pipelineName': pipeline_name, 'mode': mode}
    )


def get_pipeline(result, name):
    for pipeline_data in result.data['pipelinesOrError']['nodes']:
        if pipeline_data['name'] == name:
            return pipeline_data

    raise Exception('not found')


def test_query_multi_mode(snapshot):
    with pytest.raises(graphql.error.base.GraphQLError):
        modeless_result = execute_modes_query('multi_mode_with_resources', mode=None)

    modeless_result = execute_modes_query('multi_mode_with_resources', mode='add_mode')
    snapshot.assert_match(modeless_result.data)



from dagster_graphql.test.utils import execute_dagster_graphql
from .execution_queries import START_PIPELINE_EXECUTION_QUERY, SUBSCRIPTION_QUERY
from .setup import define_context


def sync_execute_get_payload(variables, raise_on_error=True, context=None):
    if not context:
        context = define_context(raise_on_error=raise_on_error)

    result = execute_dagster_graphql(context, START_PIPELINE_EXECUTION_QUERY, variables=variables)

    assert result.data

    if result.data['startPipelineExecution']['__typename'] != 'StartPipelineExecutionSuccess':
        raise Exception(result.data)
    run_id = result.data['startPipelineExecution']['run']['runId']

    subscription = execute_dagster_graphql(context, SUBSCRIPTION_QUERY, variables={'runId': run_id})

    subscribe_results = []
    subscription.subscribe(subscribe_results.append)

    assert len(subscribe_results) == 1
    subscribe_result = subscribe_results[0]
    assert not subscribe_result.errors
    assert subscribe_result.data
    return subscribe_result.data


def sync_execute_get_run_log_data(variables, raise_on_error=True, context=None):
    payload_data = sync_execute_get_payload(
        variables, raise_on_error=raise_on_error, context=context
    )
    assert payload_data['pipelineRunLogs']
    return payload_data['pipelineRunLogs']


def sync_execute_get_events(variables, context=None):
    return sync_execute_get_run_log_data(variables, context=context)['messages']



from dagster_graphql.test.utils import execute_dagster_graphql
from .setup import define_context

PIPELINE_OR_ERROR_SUBSET_QUERY = '''
query PipelineQuery($name: String! $solidSubset: [String!])
{
    pipelineOrError(params: { name: $name, solidSubset: $solidSubset }) {
        __typename
        ... on Pipeline {
            name
            solids {
                name
            }
            configTypes {
                __typename
                name
                ... on CompositeConfigType {
                    fields {
                    name
                    configType {
                        name
                        __typename
                    }
                    __typename
                    }
                    __typename
                }
            }
        }
        ... on SolidNotFoundError {
            solidName
        }
    }
}
'''
PIPELINE_SUBSET_QUERY = '''
query PipelineQuery($name: String! $solidSubset: [String!])
{
    pipeline(params: { name: $name, solidSubset: $solidSubset }) {
        name
        solids {
            name
        }
        configTypes {
          __typename
          name
          ... on CompositeConfigType {
            fields {
              name
              configType {
                name
                __typename
              }
              __typename
            }
            __typename
          }
        }
    }
}
'''


def field_names_of(type_dict, typename):
    return {field_data['name'] for field_data in type_dict[typename]['fields']}


def types_dict_of_result(subset_result, top_key):
    return {
        type_data['name']: type_data for type_data in subset_result.data[top_key]['configTypes']
    }


def do_test_subset(query, top_key):
    subset_result = execute_dagster_graphql(
        define_context(), query, {'name': 'csv_hello_world', 'solidSubset': ['sum_sq_solid']}
    )

    assert not subset_result.errors
    assert subset_result.data

    assert [solid_data['name'] for solid_data in subset_result.data[top_key]['solids']] == [
        'sum_sq_solid'
    ]

    subset_types_dict = types_dict_of_result(subset_result, top_key)
    assert field_names_of(subset_types_dict, 'CsvHelloWorld.SumSqSolid.Inputs') == {'sum_df'}
    assert 'CsvHelloWorld.SumSolid.Inputs' not in subset_types_dict

    full_types_dict = types_dict_of_result(
        execute_dagster_graphql(define_context(), query, {'name': 'csv_hello_world'}), top_key
    )

    assert 'CsvHelloWorld.SumSolid.Inputs' in full_types_dict
    assert 'CsvHelloWorld.SumSqSolid.Inputs' not in full_types_dict

    assert field_names_of(full_types_dict, 'CsvHelloWorld.SumSolid.Inputs') == {'num'}


def test_csv_hello_world_pipeline_subset():
    do_test_subset(PIPELINE_SUBSET_QUERY, 'pipeline')


def test_csv_hello_world_pipeline_or_error_subset():
    do_test_subset(PIPELINE_OR_ERROR_SUBSET_QUERY, 'pipelineOrError')


def test_csv_hello_world_pipeline_or_error_subset_wrong_solid_name():
    result = execute_dagster_graphql(
        define_context(),
        PIPELINE_OR_ERROR_SUBSET_QUERY,
        {'name': 'csv_hello_world', 'solidSubset': ['nope']},
    )

    assert not result.errors
    assert result.data
    assert result.data['pipelineOrError']['__typename'] == 'SolidNotFoundError'
    assert result.data['pipelineOrError']['solidName'] == 'nope'



import csv
import os
from collections import OrderedDict
from copy import deepcopy

from dagster import (
    ExecutionTargetHandle,
    InputDefinition,
    Materialization,
    OutputDefinition,
    Path,
    as_dagster_type,
    input_hydration_config,
    lambda_solid,
    output_materialization_config,
    pipeline,
)
from dagster.core.events import DagsterEventType
from dagster.core.execution.api import ExecutionSelector
from dagster.core.utils import make_new_run_id
from dagster.utils import script_relative_path
from dagster_graphql.implementation.pipeline_execution_manager import (
    MultiprocessingExecutionManager,
)
from dagster_graphql.implementation.pipeline_run_storage import (
    InMemoryPipelineRun,
    PipelineRunStatus,
)


class PoorMansDataFrame_(list):
    pass


@input_hydration_config(Path)
def df_input_schema(_context, path):
    with open(path, 'r') as fd:
        return PoorMansDataFrame_(
            [OrderedDict(sorted(x.items(), key=lambda x: x[0])) for x in csv.DictReader(fd)]
        )


@output_materialization_config(Path)
def df_output_schema(_context, path, value):
    with open(path, 'w') as fd:
        writer = csv.DictWriter(fd, fieldnames=value[0].keys())
        writer.writeheader()
        writer.writerows(rowdicts=value)

    return Materialization.file(path)


PoorMansDataFrame = as_dagster_type(
    PoorMansDataFrame_,
    input_hydration_config=df_input_schema,
    output_materialization_config=df_output_schema,
)


def get_events_of_type(events, event_type):
    return [
        event
        for event in events
        if event.is_dagster_event and event.dagster_event.event_type == event_type
    ]


def test_running():
    run_id = make_new_run_id()
    handle = ExecutionTargetHandle.for_pipeline_python_file(__file__, 'passing_pipeline')
    env_config = {
        'solids': {'sum_solid': {'inputs': {'num': script_relative_path('data/num.csv')}}}
    }
    selector = ExecutionSelector('csv_hello_world')
    pipeline_run = InMemoryPipelineRun(
        run_id,
        selector,
        env_config,
        mode='default',
        reexecution_config=None,
        step_keys_to_execute=None,
    )
    execution_manager = MultiprocessingExecutionManager()
    execution_manager.execute_pipeline(handle, passing_pipeline, pipeline_run, raise_on_error=False)
    execution_manager.join()
    assert pipeline_run.status == PipelineRunStatus.SUCCESS
    events = pipeline_run.all_logs()
    assert events

    process_start_events = get_events_of_type(events, DagsterEventType.PIPELINE_PROCESS_START)
    assert len(process_start_events) == 1

    process_started_events = get_events_of_type(events, DagsterEventType.PIPELINE_PROCESS_STARTED)
    assert len(process_started_events) == 1


def test_failing():
    run_id = make_new_run_id()
    handle = ExecutionTargetHandle.for_pipeline_python_file(__file__, 'failing_pipeline')
    env_config = {
        'solids': {'sum_solid': {'inputs': {'num': script_relative_path('data/num.csv')}}}
    }
    selector = ExecutionSelector('csv_hello_world')
    pipeline_run = InMemoryPipelineRun(
        run_id,
        selector,
        env_config,
        mode='default',
        reexecution_config=None,
        step_keys_to_execute=None,
    )
    execution_manager = MultiprocessingExecutionManager()
    execution_manager.execute_pipeline(handle, failing_pipeline, pipeline_run, raise_on_error=False)
    execution_manager.join()
    assert pipeline_run.status == PipelineRunStatus.FAILURE
    assert pipeline_run.all_logs()


def test_execution_crash():
    run_id = make_new_run_id()
    handle = ExecutionTargetHandle.for_pipeline_python_file(__file__, 'crashy_pipeline')
    env_config = {
        'solids': {'sum_solid': {'inputs': {'num': script_relative_path('data/num.csv')}}}
    }
    selector = ExecutionSelector('csv_hello_world')
    pipeline_run = InMemoryPipelineRun(
        run_id,
        selector,
        env_config,
        mode='default',
        reexecution_config=None,
        step_keys_to_execute=None,
    )
    execution_manager = MultiprocessingExecutionManager()
    execution_manager.execute_pipeline(handle, crashy_pipeline, pipeline_run, raise_on_error=False)
    execution_manager.join()
    assert pipeline_run.status == PipelineRunStatus.FAILURE
    last_log = pipeline_run.all_logs()[-1]
    print(last_log.message)
    assert last_log.message.startswith(
        'Exception: Pipeline execution process for {run_id} unexpectedly exited\n'.format(
            run_id=run_id
        )
    )


@lambda_solid(
    input_defs=[InputDefinition('num', PoorMansDataFrame)],
    output_def=OutputDefinition(PoorMansDataFrame),
)
def sum_solid(num):
    sum_df = deepcopy(num)
    for x in sum_df:
        x['sum'] = x['num1'] + x['num2']
    return PoorMansDataFrame(sum_df)


@lambda_solid(
    input_defs=[InputDefinition('sum_df', PoorMansDataFrame)],
    output_def=OutputDefinition(PoorMansDataFrame),
)
def error_solid(sum_df):  # pylint: disable=W0613
    raise Exception('foo')


@lambda_solid(
    input_defs=[InputDefinition('sum_df', PoorMansDataFrame)],
    output_def=OutputDefinition(PoorMansDataFrame),
)
def crashy_solid(sum_df):  # pylint: disable=W0613
    os._exit(1)  # pylint: disable=W0212


@pipeline
def passing_pipeline():
    return sum_solid()  # pylint: disable=no-value-for-parameter


@pipeline
def failing_pipeline():
    return error_solid(sum_solid())  # pylint: disable=no-value-for-parameter


@pipeline
def crashy_pipeline():
    crashy_solid(sum_solid())  # pylint: disable=no-value-for-parameter



from __future__ import absolute_import
from collections import namedtuple

from rx import Observable

from graphql.execution.base import ResolveInfo

from dagster import RunConfig, check

from dagster.core.events import DagsterEventType
from dagster.core.execution.api import ExecutionSelector, create_execution_plan, execute_plan
from dagster.core.execution.config import ReexecutionConfig
from dagster.core.utils import make_new_run_id

from dagster_graphql.schema.runs import from_event_record, from_dagster_event_record

from .fetch_runs import get_validated_config, validate_config
from .fetch_pipelines import get_dauphin_pipeline_from_selector
from .utils import capture_dauphin_error, UserFacingGraphQLError


@capture_dauphin_error
def start_pipeline_execution(graphene_info, execution_params, reexecution_config):
    check.inst_param(graphene_info, 'graphene_info', ResolveInfo)
    check.inst_param(execution_params, 'execution_params', ExecutionParams)
    check.opt_inst_param(reexecution_config, 'reexecution_config', ReexecutionConfig)

    pipeline_run_storage = graphene_info.context.pipeline_runs

    dauphin_pipeline = get_dauphin_pipeline_from_selector(graphene_info, execution_params.selector)

    get_validated_config(
        graphene_info,
        dauphin_pipeline,
        environment_dict=execution_params.environment_dict,
        mode=execution_params.mode,
    )

    execution_plan = create_execution_plan(
        dauphin_pipeline.get_dagster_pipeline(),
        execution_params.environment_dict,
        run_config=RunConfig(mode=execution_params.mode),
    )

    _check_start_pipeline_execution_errors(
        graphene_info, execution_params, execution_plan, reexecution_config
    )

    run = pipeline_run_storage.create_run(
        run_id=execution_params.execution_metadata.run_id
        if execution_params.execution_metadata.run_id
        else make_new_run_id(),
        selector=execution_params.selector,
        env_config=execution_params.environment_dict,
        mode=execution_params.mode,
        reexecution_config=reexecution_config,
        step_keys_to_execute=execution_params.step_keys,
    )
    pipeline_run_storage.add_run(run)

    graphene_info.context.execution_manager.execute_pipeline(
        graphene_info.context.get_handle(),
        dauphin_pipeline.get_dagster_pipeline(),
        run,
        raise_on_error=graphene_info.context.raise_on_error,
    )

    return graphene_info.schema.type_named('StartPipelineExecutionSuccess')(
        run=graphene_info.schema.type_named('PipelineRun')(run)
    )


def _check_start_pipeline_execution_errors(
    graphene_info, execution_params, execution_plan, reexecution_config
):
    if execution_params.step_keys:
        for step_key in execution_params.step_keys:
            if not execution_plan.has_step(step_key):
                raise UserFacingGraphQLError(
                    graphene_info.schema.type_named('InvalidStepError')(invalid_step_key=step_key)
                )

    if reexecution_config and reexecution_config.step_output_handles:
        for step_output_handle in reexecution_config.step_output_handles:
            if not execution_plan.has_step(step_output_handle.step_key):
                raise UserFacingGraphQLError(
                    graphene_info.schema.type_named('InvalidStepError')(
                        invalid_step_key=step_output_handle.step_key
                    )
                )

            step = execution_plan.get_step_by_key(step_output_handle.step_key)

            if not step.has_step_output(step_output_handle.output_name):
                raise UserFacingGraphQLError(
                    graphene_info.schema.type_named('InvalidOutputError')(
                        step_key=step_output_handle.step_key,
                        invalid_output_name=step_output_handle.output_name,
                    )
                )


def get_pipeline_run_observable(graphene_info, run_id, after=None):
    check.inst_param(graphene_info, 'graphene_info', ResolveInfo)
    check.str_param(run_id, 'run_id')
    check.opt_str_param(after, 'after')
    pipeline_run_storage = graphene_info.context.pipeline_runs
    run = pipeline_run_storage.get_run_by_id(run_id)

    if not run:

        def _get_error_observable(observer):
            observer.on_next(
                graphene_info.schema.type_named('PipelineRunLogsSubscriptionMissingRunIdFailure')(
                    missingRunId=run_id
                )
            )

        return Observable.create(_get_error_observable)  # pylint: disable=E1101

    def get_observable(pipeline):
        execution_plan = create_execution_plan(
            pipeline.get_dagster_pipeline(), run.config, RunConfig(mode=run.mode)
        )
        return run.observable_after_cursor(after).map(
            lambda events: graphene_info.schema.type_named('PipelineRunLogsSubscriptionSuccess')(
                runId=run_id,
                messages=[
                    from_event_record(graphene_info, event, pipeline, execution_plan)
                    for event in events
                ],
            )
        )

    return get_observable(get_dauphin_pipeline_from_selector(graphene_info, run.selector))


class ExecutionParams(
    namedtuple('_ExecutionParams', 'selector environment_dict mode execution_metadata step_keys')
):
    def __new__(cls, selector, environment_dict, mode, execution_metadata, step_keys):
        check.opt_dict_param(environment_dict, 'environment_dict', key_type=str)
        check.opt_list_param(step_keys, 'step_keys', of_type=str)

        return super(ExecutionParams, cls).__new__(
            cls,
            selector=check.inst_param(selector, 'selector', ExecutionSelector),
            environment_dict=environment_dict,
            mode=check.str_param(mode, 'mode'),
            execution_metadata=check.inst_param(
                execution_metadata, 'execution_metadata', ExecutionMetadata
            ),
            step_keys=step_keys,
        )


class ExecutionMetadata(namedtuple('_ExecutionMetadata', 'run_id tags')):
    def __new__(cls, run_id, tags):
        return super(ExecutionMetadata, cls).__new__(
            cls,
            check.opt_str_param(run_id, 'run_id'),
            check.dict_param(tags, 'tags', key_type=str, value_type=str),
        )


@capture_dauphin_error
def do_execute_plan(graphene_info, execution_params):
    check.inst_param(graphene_info, 'graphene_info', ResolveInfo)
    check.inst_param(execution_params, 'execution_params', ExecutionParams)

    return _execute_plan_resolve_config(
        graphene_info,
        execution_params,
        get_dauphin_pipeline_from_selector(graphene_info, execution_params.selector),
    )


def _execute_plan_resolve_config(graphene_info, execution_params, dauphin_pipeline):
    check.inst_param(graphene_info, 'graphene_info', ResolveInfo)
    check.inst_param(execution_params, 'execution_params', ExecutionParams)
    validate_config(
        graphene_info, dauphin_pipeline, execution_params.environment_dict, execution_params.mode
    )
    return _do_execute_plan(graphene_info, execution_params, dauphin_pipeline)


def _do_execute_plan(graphene_info, execution_params, dauphin_pipeline):
    check.inst_param(graphene_info, 'graphene_info', ResolveInfo)
    check.inst_param(execution_params, 'execution_params', ExecutionParams)

    event_records = []

    run_config = RunConfig(
        run_id=execution_params.execution_metadata.run_id,
        mode=execution_params.mode,
        tags=execution_params.execution_metadata.tags,
        event_callback=event_records.append,
    )

    execution_plan = create_execution_plan(
        pipeline=dauphin_pipeline.get_dagster_pipeline(),
        environment_dict=execution_params.environment_dict,
        run_config=run_config,
    )

    if execution_params.step_keys:
        for step_key in execution_params.step_keys:
            if not execution_plan.has_step(step_key):
                raise UserFacingGraphQLError(
                    graphene_info.schema.type_named('InvalidStepError')(invalid_step_key=step_key)
                )

    execute_plan(
        execution_plan=execution_plan,
        environment_dict=execution_params.environment_dict,
        run_config=run_config,
        step_keys_to_execute=execution_params.step_keys,
    )

    def to_graphql_event(event_record):
        return from_dagster_event_record(
            graphene_info, event_record, dauphin_pipeline, execution_plan
        )

    return graphene_info.schema.type_named('ExecutePlanSuccess')(
        pipeline=dauphin_pipeline,
        has_failures=any(
            er
            for er in event_records
            if er.is_dagster_event and er.dagster_event.event_type == DagsterEventType.STEP_FAILURE
        ),
        step_events=list(
            map(to_graphql_event, filter(lambda er: er.is_dagster_event, event_records))
        ),
    )



from .utils import sync_execute_get_events


def test_materializations(snapshot):
    logs = sync_execute_get_events(
        variables={
            'executionParams': {'selector': {'name': 'materialization_pipeline'}, 'mode': 'default'}
        }
    )
    materializations = [log for log in logs if log['__typename'] == 'StepMaterializationEvent']
    assert len(materializations) == 1
    mat = materializations[0]['materialization']
    assert mat['label'] == 'all_types'

    text_entry = mat['metadataEntries'][0]
    assert text_entry['__typename'] == 'EventTextMetadataEntry'
    assert text_entry['text']

    text_entry = mat['metadataEntries'][1]
    assert text_entry['__typename'] == 'EventUrlMetadataEntry'
    assert text_entry['url']

    text_entry = mat['metadataEntries'][2]
    assert text_entry['__typename'] == 'EventPathMetadataEntry'
    assert text_entry['path']

    text_entry = mat['metadataEntries'][3]
    assert text_entry['__typename'] == 'EventJsonMetadataEntry'
    assert text_entry['jsonString']

    snapshot.assert_match(logs)



# -*- coding: utf-8 -*-
# snapshottest: v1 - https://goo.gl/zC4yUc
from __future__ import unicode_literals

from snapshottest import Snapshot


snapshots = Snapshot()

snapshots['test_successful_enviroment_schema 1'] = {
    'environmentSchemaOrError': {
        '__typename': 'EnvironmentSchema',
        'allConfigTypes': [
            {
                'name': None
            },
            {
                'name': None
            },
            {
                'name': None
            },
            {
                'name': None
            },
            {
                'name': None
            },
            {
                'name': None
            },
            {
                'name': None
            },
            {
                'name': None
            },
            {
                'name': None
            },
            {
                'name': None
            },
            {
                'name': None
            },
            {
                'name': None
            },
            {
                'name': None
            },
            {
                'name': None
            },
            {
                'name': None
            },
            {
                'name': None
            },
            {
                'name': None
            },
            {
                'name': None
            },
            {
                'name': None
            },
            {
                'name': None
            },
            {
                'name': None
            },
            {
                'name': None
            },
            {
                'name': None
            },
            {
                'name': None
            },
            {
                'name': 'Any'
            },
            {
                'name': 'Any.InputHydrationConfig'
            },
            {
                'name': 'Any.MaterializationSchema'
            },
            {
                'name': 'Bool'
            },
            {
                'name': 'Bool.InputHydrationConfig'
            },
            {
                'name': 'Bool.MaterializationSchema'
            },
            {
                'name': 'Float'
            },
            {
                'name': 'Float.InputHydrationConfig'
            },
            {
                'name': 'Float.MaterializationSchema'
            },
            {
                'name': 'Int'
            },
            {
                'name': 'Int.InputHydrationConfig'
            },
            {
                'name': 'Int.MaterializationSchema'
            },
            {
                'name': 'MultiModeWithResources.AddMode.StorageConfig'
            },
            {
                'name': 'MultiModeWithResources.AddMode.StorageConfig.Filesystem'
            },
            {
                'name': 'MultiModeWithResources.AddMode.StorageConfig.InMemory'
            },
            {
                'name': 'MultiModeWithResources.ApplyToThree.Outputs'
            },
            {
                'name': 'MultiModeWithResources.ExecutionConfig'
            },
            {
                'name': 'MultiModeWithResources.ExpectationsConfig'
            },
            {
                'name': 'MultiModeWithResources.LoggerConfig'
            },
            {
                'name': 'MultiModeWithResources.LoggerConfig.Console'
            },
            {
                'name': 'MultiModeWithResources.Mode.AddMode.Environment'
            },
            {
                'name': 'MultiModeWithResources.Mode.AddMode.Resources'
            },
            {
                'name': 'MultiModeWithResources.Mode.AddMode.Resources.Op'
            },
            {
                'name': 'MultiModeWithResources.SolidConfig.ApplyToThree'
            },
            {
                'name': 'MultiModeWithResources.SolidsConfigDictionary'
            },
            {
                'name': 'Path'
            },
            {
                'name': 'Path.MaterializationSchema'
            },
            {
                'name': 'String'
            },
            {
                'name': 'String.InputHydrationConfig'
            },
            {
                'name': 'String.MaterializationSchema'
            }
        ],
        'rootEnvironmentType': {
            'name': 'MultiModeWithResources.Mode.AddMode.Environment'
        }
    }
}

snapshots['test_success_config_type_fetch 1'] = {
    'environmentSchemaOrError': {
        '__typename': 'EnvironmentSchema',
        'configTypeOrError': {
            '__typename': 'CompositeConfigType',
            'name': 'MultiModeWithResources.Mode.AddMode.Environment'
        }
    }
}

snapshots['test_success_config_type_not_found 1'] = {
    'environmentSchemaOrError': {
        '__typename': 'EnvironmentSchema',
        'configTypeOrError': {
            '__typename': 'ConfigTypeNotFoundError'
        }
    }
}

snapshots['test_basic_valid_config_on_environment_schema 1'] = {
    'environmentSchemaOrError': {
        'isEnvironmentConfigValid': {
            '__typename': 'PipelineConfigValidationValid',
            'pipeline': {
                'name': 'csv_hello_world'
            }
        }
    }
}

snapshots['test_basic_invalid_config_on_environment_schema 1'] = {
    'environmentSchemaOrError': {
        'isEnvironmentConfigValid': {
            '__typename': 'PipelineConfigValidationInvalid',
            'errors': [
                {
                    '__typename': 'FieldNotDefinedConfigError',
                    'fieldName': 'nope',
                    'message': 'Field "nope" is not defined at document config root. Expected: "{ execution?: CsvHelloWorld.ExecutionConfig expectations?: CsvHelloWorld.ExpectationsConfig loggers?: CsvHelloWorld.LoggerConfig resources?: CsvHelloWorld.Mode.Default.Resources solids: CsvHelloWorld.SolidsConfigDictionary storage?: CsvHelloWorld.Default.StorageConfig }"',
                    'reason': 'FIELD_NOT_DEFINED',
                    'stack': {
                        'entries': [
                        ]
                    }
                },
                {
                    '__typename': 'MissingFieldConfigError',
                    'field': {
                        'name': 'solids'
                    },
                    'message': 'Missing required field "solids" at document config root. Available Fields: "[\'execution\', \'expectations\', \'loggers\', \'resources\', \'solids\', \'storage\']".',
                    'reason': 'MISSING_REQUIRED_FIELD',
                    'stack': {
                        'entries': [
                        ]
                    }
                }
            ],
            'pipeline': {
                'name': 'csv_hello_world'
            }
        }
    }
}



from dagster import check

from dagster.core.execution.api import ExecutionSelector
from dagster.core.definitions import create_environment_schema
from dagster_graphql.schema.config_types import to_dauphin_config_type
from dagster_graphql.schema.runtime_types import to_dauphin_runtime_type

from .fetch_pipelines import get_dagster_pipeline_from_selector
from .utils import UserFacingGraphQLError, capture_dauphin_error


@capture_dauphin_error
def get_config_type(graphene_info, pipeline_name, config_type_name, mode):
    check.str_param(pipeline_name, 'pipeline_name')
    check.str_param(config_type_name, 'config_type_name')
    check.opt_str_param(mode, 'mode')

    pipeline = get_dagster_pipeline_from_selector(graphene_info, ExecutionSelector(pipeline_name))
    environment_schema = create_environment_schema(pipeline, mode)
    if not environment_schema.has_config_type(config_type_name):
        raise UserFacingGraphQLError(
            graphene_info.schema.type_named('ConfigTypeNotFoundError')(
                pipeline=pipeline, config_type_name=config_type_name
            )
        )

    return to_dauphin_config_type(environment_schema.config_type_named(config_type_name))


@capture_dauphin_error
def get_runtime_type(graphene_info, pipeline_name, type_name):
    pipeline = get_dagster_pipeline_from_selector(graphene_info, ExecutionSelector(pipeline_name))

    if not pipeline.has_runtime_type(type_name):
        raise UserFacingGraphQLError(
            graphene_info.schema.type_named('RuntimeTypeNotFoundError')(
                pipeline=pipeline, runtime_type_name=type_name
            )
        )

    return to_dauphin_runtime_type(pipeline.runtime_type_named(type_name))



PRODUCTION_QUERY = '''
query AppQuery {
  pipelinesOrError {
    ... on Error {
      message
      stack
      __typename
    }
    ... on PipelineConnection {
      nodes {
        ...PipelineFragment
        __typename
      }
    }
    __typename
  }
}

fragment PipelineFragment on Pipeline {
  name
  description
  solids {
    ...SolidFragment
    __typename
  }
  modes {
    name
    description
    resources {
      name
      configField {
        ...ConfigFieldFragment
        __typename
      }
    }
  }
  ...PipelineGraphFragment
  __typename
}

fragment SolidFragment on Solid {
  ...SolidTypeSignatureFragment
  name
  definition {
    description
    metadata {
      key
      value
      __typename
    }
    ... on SolidDefinition {
      configDefinition {
        ...ConfigFieldFragment
        __typename
      }
    }
    __typename
  }
  inputs {
    definition {
      name
      description
      type {
        ... RuntimeTypeWithTooltipFragment
        __typename
      }
      expectations {
        name
        description
        __typename
      }
      __typename
    }
    dependsOn {
      definition {
        name
        __typename
      }
      solid {
        name
        __typename
      }
      __typename
    }
    __typename
  }
  outputs {
    definition {
      name
      description
      type {
        __typename
      }
      expectations {
        name
        description
        __typename
      }
      expectations {
        name
        description
        __typename
      }
      __typename
    }
    __typename
  }
  __typename
}

fragment RuntimeTypeWithTooltipFragment on RuntimeType {
  name
  description
  __typename
}

fragment SolidTypeSignatureFragment on Solid {
  outputs {
    definition {
      name
      type {
        ... RuntimeTypeWithTooltipFragment
        __typename
      }
      __typename
    }
    __typename
  }
  inputs {
    definition {
      name
      type {
        ... RuntimeTypeWithTooltipFragment
        __typename
      }
      __typename
    }
    __typename
  }
  __typename
}

fragment ConfigFieldFragment on ConfigTypeField {
  configType {
    __typename
    name
    description
    ... on CompositeConfigType {
      fields {
        name
        description
        isOptional
        defaultValue
        configType {
          name
          description
          ... on CompositeConfigType {
            fields {
              name
              description
              isOptional
              defaultValue
              configType {
                name
                description
                __typename
              }
              __typename
            }
            __typename
          }
          __typename
        }
        __typename
      }
      __typename
    }
  }
  __typename
}

fragment PipelineGraphFragment on Pipeline {
  name
  solids {
    ...SolidNodeFragment
    __typename
  }
  __typename
}

fragment SolidNodeFragment on Solid {
  name
  inputs {
    definition {
      name
      type {
        name
        __typename
      }
      __typename
    }
    dependsOn {
      definition {
        name
        __typename
      }
      solid {
        name
        __typename
      }
      __typename
    }
    __typename
  }
  outputs {
    definition {
      name
      type {
        name
        __typename
      }
      expectations {
        name
        description
        __typename
      }
      __typename
    }
    __typename
  }
  __typename
}
'''



import copy

from dagster_graphql.test.utils import execute_dagster_graphql

from .utils import define_context, sync_execute_get_run_log_data

try:
    # Python 2 tempfile doesn't have tempfile.TemporaryDirectory
    import backports.tempfile as tempfile
except ImportError:
    import tempfile


RUNS_QUERY = '''
query PipelineRunsRootQuery($name: String!) {
  pipeline(params: { name: $name }) {
    name
    runs {
      ...RunHistoryRunFragment
    }
  }
}

fragment RunHistoryRunFragment on PipelineRun {
  runId
  status
  pipeline {
    name
  }
  logs {
    nodes {
      __typename
      ... on MessageEvent {
        timestamp
      }
    }
  }
  executionPlan {
    steps {
      key
    }
  }
  environmentConfigYaml
  mode
}
'''


def _get_runs_data(result, run_id):
    for run_data in result.data['pipeline']['runs']:
        if run_data['runId'] == run_id:
            # so caller can delete keys
            return copy.deepcopy(run_data)

    raise Exception('nope')


def test_get_runs_over_graphql(snapshot):
    context = define_context()
    payload_one = sync_execute_get_run_log_data(
        {
            'executionParams': {
                'selector': {'name': 'multi_mode_with_resources'},
                'mode': 'add_mode',
                'environmentConfigData': {'resources': {'op': {'config': 2}}},
            }
        },
        context=context,
    )
    run_id_one = payload_one['runId']

    payload_two = sync_execute_get_run_log_data(
        {
            'executionParams': {
                'selector': {'name': 'multi_mode_with_resources'},
                'mode': 'add_mode',
                'environmentConfigData': {'resources': {'op': {'config': 3}}},
            }
        },
        #
        #     'pipeline': {'name': 'multi_mode_with_resources'},
        #     'mode': 'add_mode',
        #     'environmentConfigData': {'resources': {'op': {'config': 3}}},
        # },
        context=context,
    )

    run_id_two = payload_two['runId']

    result = execute_dagster_graphql(
        context, RUNS_QUERY, variables={'name': 'multi_mode_with_resources'}
    )

    run_one_data = _get_runs_data(result, run_id_one)
    assert run_one_data['logs']
    del run_one_data['logs']  # unstable between invokes
    del run_one_data['runId']  # unstable between invokes
    snapshot.assert_match(run_one_data)

    run_two_data = _get_runs_data(result, run_id_two)
    assert run_two_data['logs']
    del run_two_data['logs']  # unstable between invokes
    del run_two_data['runId']  # unstable between invokes
    snapshot.assert_match(run_two_data)


def test_persisted_runs_over_graphql():
    with tempfile.TemporaryDirectory() as log_dir:
        write_context = define_context(log_dir=log_dir)
        payload_one = sync_execute_get_run_log_data(
            {
                'executionParams': {
                    'selector': {'name': 'multi_mode_with_resources'},
                    'mode': 'add_mode',
                    'environmentConfigData': {'resources': {'op': {'config': 2}}},
                }
            },
            context=write_context,
        )
        run_id_one = payload_one['runId']

        payload_two = sync_execute_get_run_log_data(
            {
                'executionParams': {
                    'selector': {'name': 'multi_mode_with_resources'},
                    'mode': 'add_mode',
                    'environmentConfigData': {'resources': {'op': {'config': 3}}},
                }
            },
            context=write_context,
        )

        run_id_two = payload_two['runId']

        read_context = define_context(log_dir=log_dir)

        result = execute_dagster_graphql(
            read_context, RUNS_QUERY, variables={'name': 'multi_mode_with_resources'}
        )

        run_one_data = _get_runs_data(result, run_id_one)
        assert [log['__typename'] for log in run_one_data['logs']['nodes']] == [
            msg['__typename'] for msg in payload_one['messages']
        ]

        run_two_data = _get_runs_data(result, run_id_two)
        assert [log['__typename'] for log in run_two_data['logs']['nodes']] == [
            msg['__typename'] for msg in payload_two['messages']
        ]



from graphql.execution.base import ResolveInfo

from dagster import check
from dagster.core.execution.api import ExecutionSelector
from dagster.core.definitions.environment_schema import create_environment_schema, EnvironmentSchema
from dagster.core.definitions.pipeline import PipelineDefinition
from dagster.core.types.evaluator import evaluate_config
from .fetch_pipelines import get_dagster_pipeline_from_selector
from .utils import capture_dauphin_error, UserFacingGraphQLError


@capture_dauphin_error
def resolve_environment_schema_or_error(graphene_info, selector, mode):
    check.inst_param(graphene_info, 'graphene_info', ResolveInfo)
    check.inst_param(selector, 'selector', ExecutionSelector)

    dagster_pipeline = get_dagster_pipeline_from_selector(graphene_info, selector)

    if not dagster_pipeline.has_mode_definition(mode):
        raise UserFacingGraphQLError(
            graphene_info.schema.type_named('ModeNotFoundError')(mode=mode, selector=selector)
        )

    return graphene_info.schema.type_named('EnvironmentSchema')(
        dagster_pipeline=dagster_pipeline,
        environment_schema=create_environment_schema(dagster_pipeline, mode),
    )


@capture_dauphin_error
def resolve_config_type_or_error(
    graphene_info, environment_schema, dagster_pipeline, config_type_name
):
    from dagster_graphql.schema.config_types import to_dauphin_config_type

    check.inst_param(graphene_info, 'graphene_info', ResolveInfo)
    check.inst_param(environment_schema, 'environment_scheam', EnvironmentSchema)
    check.inst_param(dagster_pipeline, 'dagster_pipeline', PipelineDefinition)
    check.str_param(config_type_name, 'config_type_name')

    if not environment_schema.has_config_type(config_type_name):
        raise UserFacingGraphQLError(
            graphene_info.schema.type_named('ConfigTypeNotFoundError')(
                pipeline=dagster_pipeline, config_type_name=config_type_name
            )
        )

    return to_dauphin_config_type(environment_schema.config_type_named(config_type_name))


@capture_dauphin_error
def resolve_is_environment_config_valid(
    graphene_info, environment_schema, dagster_pipeline, environment_dict
):
    check.inst_param(graphene_info, 'graphene_info', ResolveInfo)
    check.inst_param(environment_schema, 'environment_schema', EnvironmentSchema)
    check.inst_param(dagster_pipeline, 'dagster_pipeline', PipelineDefinition)
    check.opt_dict_param(environment_dict, 'environment_dict', key_type=str)

    validated_config = evaluate_config(
        environment_schema.environment_type, environment_dict, dagster_pipeline
    )

    dauphin_pipeline = graphene_info.schema.type_named('Pipeline')(dagster_pipeline)

    if not validated_config.success:
        raise UserFacingGraphQLError(
            graphene_info.schema.type_named('PipelineConfigValidationInvalid')(
                pipeline=dauphin_pipeline,
                errors=[
                    graphene_info.schema.type_named(
                        'PipelineConfigValidationError'
                    ).from_dagster_error(graphene_info, err)
                    for err in validated_config.errors
                ],
            )
        )

    return graphene_info.schema.type_named('PipelineConfigValidationValid')(dauphin_pipeline)



# -*- coding: utf-8 -*-
# snapshottest: v1 - https://goo.gl/zC4yUc
from __future__ import unicode_literals

from snapshottest import Snapshot


snapshots = Snapshot()

snapshots['test_materializations 1'] = [
    {
        '__typename': 'PipelineStartEvent',
        'level': 'DEBUG',
        'message': 'DagsterEventType.PIPELINE_START for pipeline materialization_pipeline',
        'step': None
    },
    {
        '__typename': 'ExecutionStepStartEvent',
        'level': 'DEBUG',
        'message': 'DagsterEventType.STEP_START for step materialize.compute',
        'step': {
            'key': 'materialize.compute',
            'kind': 'COMPUTE',
            'solidHandleID': 'materialize'
        }
    },
    {
        '__typename': 'LogMessageEvent',
        'level': 'INFO',
        'message': "Solid 'materialize' materialized 'all_types'",
        'step': {
            'key': 'materialize.compute',
            'solidHandleID': 'materialize'
        }
    },
    {
        '__typename': 'StepMaterializationEvent',
        'level': 'DEBUG',
        'materialization': {
            'description': 'a materialization with all metadata types',
            'label': 'all_types',
            'metadataEntries': [
                {
                    '__typename': 'EventTextMetadataEntry',
                    'description': None,
                    'label': 'text',
                    'text': 'text is cool'
                },
                {
                    '__typename': 'EventUrlMetadataEntry',
                    'description': None,
                    'label': 'url',
                    'url': 'https://bigty.pe/neato'
                },
                {
                    '__typename': 'EventPathMetadataEntry',
                    'description': None,
                    'label': 'path',
                    'path': '/tmp/awesome'
                },
                {
                    '__typename': 'EventJsonMetadataEntry',
                    'description': None,
                    'jsonString': '{"is_dope": true}',
                    'label': 'json'
                }
            ]
        },
        'message': 'DagsterEventType.STEP_MATERIALIZATION for step materialize.compute',
        'step': {
            'key': 'materialize.compute',
            'solidHandleID': 'materialize'
        }
    },
    {
        '__typename': 'LogMessageEvent',
        'level': 'INFO',
        'message': "Solid 'materialize' emitted output 'result' value None",
        'step': {
            'key': 'materialize.compute',
            'solidHandleID': 'materialize'
        }
    },
    {
        '__typename': 'ExecutionStepOutputEvent',
        'level': 'DEBUG',
        'message': 'DagsterEventType.STEP_OUTPUT for step materialize.compute',
        'outputName': 'result',
        'step': {
            'key': 'materialize.compute',
            'kind': 'COMPUTE',
            'solidHandleID': 'materialize'
        },
        'typeCheck': {
            'description': None,
            'label': 'result',
            'metadataEntries': [
            ]
        },
        'valueRepr': 'None'
    },
    {
        '__typename': 'ExecutionStepSuccessEvent',
        'level': 'DEBUG',
        'message': 'DagsterEventType.STEP_SUCCESS for step materialize.compute',
        'step': {
            'key': 'materialize.compute',
            'solidHandleID': 'materialize'
        }
    },
    {
        '__typename': 'PipelineSuccessEvent',
        'level': 'DEBUG',
        'message': 'DagsterEventType.PIPELINE_SUCCESS for pipeline materialization_pipeline',
        'step': None
    }
]



from .utils import execute_dagster_graphql


PRESETS_QUERY = '''
query PresetsQuery($name: String!) {
    pipeline(params: { name: $name }) {
        name
        presets {
            __typename
            name
            solidSubset
            environmentConfigYaml
            mode
        }
    }
}  
'''


def execute_preset_query(pipeline_name, context):
    return execute_dagster_graphql(context, PRESETS_QUERY, variables={'name': pipeline_name})



import json

from click.testing import CliRunner

from dagster import (
    DependencyDefinition,
    InputDefinition,
    Int,
    OutputDefinition,
    PipelineDefinition,
    RepositoryDefinition,
    lambda_solid,
    seven,
)
from dagster.utils import script_relative_path

from dagster_graphql.cli import ui


@lambda_solid(input_defs=[InputDefinition('num', Int)], output_def=OutputDefinition(Int))
def add_one(num):
    return num + 1


@lambda_solid(input_defs=[InputDefinition('num', Int)], output_def=OutputDefinition(Int))
def mult_two(num):
    return num * 2


def define_csv_hello_world():
    return PipelineDefinition(
        name='math',
        solid_defs=[add_one, mult_two],
        dependencies={'add_one': {}, 'mult_two': {'num': DependencyDefinition(add_one.name)}},
    )


def define_repository():
    return RepositoryDefinition(name='test', pipeline_dict={'math': define_csv_hello_world})


def test_basic_introspection():
    query = '{ __schema { types { name } } }'

    repo_path = script_relative_path('./cli_test_repository.yaml')

    runner = CliRunner()
    result = runner.invoke(ui, ['-y', repo_path, query])

    assert result.exit_code == 0

    result_data = json.loads(result.output)
    assert result_data['data']


def test_basic_pipelines():
    query = '{ pipelines { nodes { name } } }'

    repo_path = script_relative_path('./cli_test_repository.yaml')

    runner = CliRunner()
    result = runner.invoke(ui, ['-y', repo_path, query])

    assert result.exit_code == 0

    result_data = json.loads(result.output)
    assert result_data['data']


def test_basic_variables():
    query = 'query FooBar($pipelineName: String!){ pipeline(params:{name: $pipelineName}){ name} }'
    variables = '{"pipelineName": "math"}'
    repo_path = script_relative_path('./cli_test_repository.yaml')

    runner = CliRunner()
    result = runner.invoke(ui, ['-y', repo_path, '-v', variables, query])

    assert result.exit_code == 0

    result_data = json.loads(result.output)
    assert result_data['data']


START_PIPELINE_EXECUTION_QUERY = '''
mutation ($executionParams: ExecutionParams!) {
    startPipelineExecution(executionParams: $executionParams) {
        __typename
        ... on StartPipelineExecutionSuccess {
            run {
                runId
                pipeline { name }
                logs {
                    nodes {
                        __typename
                        ... on MessageEvent {
                            message
                            level
                        }
                        ... on ExecutionStepStartEvent {
                            step { kind }
                        }
                    }
                }
            }
        }
        ... on PipelineConfigValidationInvalid {
            pipeline { name }
            errors { message }
        }
        ... on PipelineNotFoundError {
            pipelineName
        }
    }
}
'''


def test_start_execution():
    variables = seven.json.dumps(
        {
            'executionParams': {
                'selector': {'name': 'math'},
                'environmentConfigData': {
                    'solids': {'add_one': {'inputs': {'num': {'value': 123}}}}
                },
                'mode': 'default',
            }
        }
    )

    repo_path = script_relative_path('./repository.yaml')

    runner = CliRunner()
    result = runner.invoke(ui, ['-y', repo_path, '-v', variables, START_PIPELINE_EXECUTION_QUERY])

    assert result.exit_code == 0

    try:
        result_data = json.loads(result.output.strip('\n').split('\n')[-1])
        assert result_data['data']
    except Exception as e:
        raise Exception('Failed with {} Exception: {}'.format(result.output, e))



FRAGMENTS = '''
fragment metadataEntryFragment on EventMetadataEntry {
    label
    description
    ... on EventPathMetadataEntry {
        path
    }
    ... on EventJsonMetadataEntry {
        jsonString
    }
    ... on EventTextMetadataEntry {
        text
    }
    ... on EventUrlMetadataEntry {
        url
    }
}

fragment stepEventFragment on StepEvent {
    ... on ExecutionStepStartEvent {
        step { kind }
    }
    ... on ExecutionStepFailureEvent {
        step { key kind }
        error {
            message
        }
        failureMetadata {
            label
            description
            metadataEntries {
                ...metadataEntryFragment
            }
        }
    }
    ... on ExecutionStepInputEvent {
        step { key kind }
        inputName
        typeCheck {
            label
            description
            metadataEntries {
                ...metadataEntryFragment
            }
        }
    }
    ... on ExecutionStepOutputEvent {
        step { key kind }
        outputName
        typeCheck {
            label
            description
            metadataEntries {
                ...metadataEntryFragment
            }
        }
    }
    ... on StepExpectationResultEvent {
        expectationResult {
            success
            label
            description
            metadataEntries {
                ...metadataEntryFragment
            }
        }
    }
}

'''
START_PIPELINE_EXECUTION_QUERY = (
    FRAGMENTS
    + '''

mutation (
    $executionParams: ExecutionParams!
    $reexecutionConfig: ReexecutionConfig
) {
    startPipelineExecution(
        executionParams: $executionParams
        reexecutionConfig: $reexecutionConfig
    ) {
        __typename
        ... on StartPipelineExecutionSuccess {
            run {
                runId
                pipeline { name }
                logs {
                    nodes {
                        __typename
                        ... on MessageEvent  {
                            message
                            level
                        }
                        ...stepEventFragment
                    }
                }
            }
        }
        ... on PipelineConfigValidationInvalid {
            pipeline { name }
            errors { message }
        }
        ... on PipelineNotFoundError {
            pipelineName
        }
    }
}
'''
)


START_PIPELINE_EXECUTION_SNAPSHOT_QUERY = (
    FRAGMENTS
    + '''
mutation (
    $executionParams: ExecutionParams!
    $reexecutionConfig: ReexecutionConfig
) {
    startPipelineExecution(
        executionParams: $executionParams
        reexecutionConfig: $reexecutionConfig
    ) {
        __typename
        ... on StartPipelineExecutionSuccess {
            run {
                pipeline { name }
                logs {
                    nodes {
                        __typename
                        ... on MessageEvent {
                            level
                        }
                        ...stepEventFragment
                    }
                }
            }
        }
        ... on PipelineConfigValidationInvalid {
            pipeline { name }
            errors { message }
        }
        ... on PipelineNotFoundError {
            pipelineName
        }
        ... on InvalidStepError {
            invalidStepKey
        }
        ... on InvalidOutputError {
            stepKey
            invalidOutputName
        }
    }
}
'''
)

SUBSCRIPTION_QUERY = (
    FRAGMENTS
    + '''
subscription subscribeTest($runId: ID!) {
    pipelineRunLogs(runId: $runId) {
        __typename
        ... on PipelineRunLogsSubscriptionSuccess {
            runId,
            messages {
                __typename
                ...stepEventFragment

                ... on ExecutionStepOutputEvent {
                    valueRepr
                }

                ... on MessageEvent {
                    message
                    step { key solidHandleID }
                    level
                }

                # only include here because unstable between runs
                ... on StepMaterializationEvent {
                    materialization {
                        label
                        description
                        metadataEntries {
                            __typename
                            ...metadataEntryFragment
                        }
                    }
                }
                ... on ExecutionStepFailureEvent {
                    step { key kind }
                    error {
                        message
                        stack
                    }
                }
            }
        }
        ... on PipelineRunLogsSubscriptionMissingRunIdFailure {
            missingRunId
        }
    }
}
'''
)


PIPELINE_REEXECUTION_INFO_QUERY = '''
query ReexecutionInfoQuery($runId: ID!) {
  pipelineRunOrError(runId: $runId) {
    __typename
    ... on PipelineRun {
        stepKeysToExecute
      }
    }
  }
'''
